
#ifndef __DBELEMENTTOKEN__
#define __DBELEMENTTOKEN__

//
// For class definiton of TAbstractDesignator
//
#include "AbstractDesignator.h"

//
// For AConst<> and AnUpdate<>
//
#include "ReferenceTemplates.h"

class TDBElement;
class TTransaction;

class TDBElementToken : public TAbstractDesignator
{
private:
	AConst<TDBElement>		fRecord;

public:
	DeclareSmallClassData(TDBElementToken, TAbstractDesignator);

	TDBElementToken(AConst<TDBElement> record);
	virtual ~TDBElementToken();

	virtual void						CloneOwnedObjects();
	
	virtual DescType					ObjectClass(TTransaction* t, Boolean recordedClass = false);
	virtual Boolean						DerivedFromOSLClass(TTransaction* t, DescType objectClass);

	virtual TAbstractScriptableObject*	ParentObject(TTransaction* t);
	virtual TAbstractObjectIterator*	ElementIterator(TTransaction* t);

	virtual TAbstractScriptableObject*	AccessByProperty(TTransaction* t, DescType propertyIdentifier);

	// These accessors are used less frequently:
	
	virtual TAbstractScriptableObject*	AccessByUniqueID(TTransaction* t, DescType desiredClass, TDescriptor uniqueID);
	
	// Methods relating to the data types accessible to this object
	
	virtual DescType					BestType(TTransaction* t, DescType propertyName);
	virtual DescType					DefaultType(TTransaction* t, DescType propertyName);
	virtual Boolean						CanReturnDataOfType(TTransaction* t, DescType propertyName, DescType desiredType);

	// Methods to override to provide data access to this object
	
	virtual TDescriptor					GetProperty(TTransaction* t, DescType propertyName, DescType desiredType, unsigned long additionalInfo = 0);
	virtual void						SetProperty(TTransaction* t, DescType propertyName, TDescriptor& data, unsigned long additionalInfo = 0);
	
	// Make an object specifier for this object
	
	virtual TDescriptor					BuildObjectSpecifier(TTransaction* t);
	virtual void						MakeKeyDataForSelf(TTransaction* t, DescType& keyForm, TDescriptor& keyData);
	virtual TDescriptor					BuildSpecifierForParent(TTransaction* t);

	// Event handlers
	
	virtual TDescriptor					AECommand(TTransaction* t, TAEvent ae, TAEvent reply, long aeCommandID, TAbstractScriptableObject* auxObjects = nil, long auxInfo = 0);
	virtual TAbstractScriptableObject*	CreateNewElement(TTransaction* t, TAEvent ae, TAEvent reply, DescType newObjectClass, TDescriptor initialData, TDescriptor initialProperties, Boolean& usedInitialData, Boolean& usedInitialProperties);
		
	// Compare a property of this object with a descriptor
	
	virtual Boolean						CompareProperty(TTransaction* t, DescType propertyIdentifier, DescType comparisonOperator, TDescriptor compareWith);
	
	virtual Boolean						ObjectsAreTheSame(TTransaction* t, TAbstractScriptableObject* objectToTest);

	Boolean								IsMetaRoot();
	long								RecordIndex();
	Int64								DocumentIdentifier();
	
	//
	// Used to redefine this token to represent a different database element
	//
	void								AssignDifferentElement(AConst<TDBElement> record) { ASSERT(fReferenceCount == 0); fRecord = record; }
};

#endif
