
#include "DBElementTokenIterator.h"
#include "DBElementToken.h"
#include "DBElement.h"
#include "AbstractSearchSpec.h"
#include "AbstractCollector.h"

//--------------------------------------------------------------------------------
// TDBElementTokenIterator::~TDBElementTokenIterator
//--------------------------------------------------------------------------------
TDBElementTokenIterator::~TDBElementTokenIterator()
{
} // TDBElementTokenIterator::~TDBElementTokenIterator

//--------------------------------------------------------------------------------
// TDBElementTokenIterator::Reset
//--------------------------------------------------------------------------------
void TDBElementTokenIterator::Reset(TTransaction*, Boolean iterationDirection /* = kForwardIteration */)
{
	fIterator.Reset(iterationDirection);
}

//--------------------------------------------------------------------------------
// TDBElementTokenIterator::More
//--------------------------------------------------------------------------------
Boolean TDBElementTokenIterator::More(TTransaction*) const
{
	return fIterator.More();
}

//--------------------------------------------------------------------------------
// TDBElementTokenIterator::Next
//--------------------------------------------------------------------------------
void TDBElementTokenIterator::Next(TTransaction*)
{
	fIterator.Next();
}

//--------------------------------------------------------------------------------
// TDBElementTokenIterator::Current
//--------------------------------------------------------------------------------
TAbstractScriptableObject* TDBElementTokenIterator::Current(TTransaction*)
{
	return new TDBElementToken(fIterator.Current()->DBElementCursor());
}

//--------------------------------------------------------------------------------
// TDBElementTokenIterator::CurrentDerivedFromOSLClass
//--------------------------------------------------------------------------------
Boolean TDBElementTokenIterator::CurrentDerivedFromOSLClass(TTransaction* t, DescType objectClass)
{
	return TDBElementToken(fIterator.Current()->DBElementCursor()).DerivedFromOSLClass(t, objectClass);
}

//--------------------------------------------------------------------------------
// TDBElementTokenIterator::GetNamedElement
//--------------------------------------------------------------------------------
TAbstractScriptableObject* TDBElementTokenIterator::GetNamedElement(TTransaction* t, DescType desiredClass, TDescriptor nameDesc)
{
	TAbstractScriptableObject* result = nil;
	
	short oldState = nameDesc.Lock();
	
	//
	// A record iterator is given a reference to the top element of the tree
	// to be iterated over, so we must ask this item's _parent_ to find
	// the element with the name we want. 
	//
	AConst<TDBElement> namedElement = fIteratorOwner->DBElementCursor()->TreeOwner(t)->DBElementCursor()->GetNamedElement(t, nameDesc);
	nameDesc.Unlock(oldState);
	if(namedElement.Exists())
	{
		result = new TDBElementToken(namedElement);
		if(result->DerivedFromOSLClass(t, desiredClass) == false)
		{
			delete result;
			result = nil;
		}
	}
	
	return result;
} // TDBElementTokenIterator::GetNamedElement

#if 0

//--------------------------------------------------------------------------------
// TDBElementTokenIterator::Contains
//--------------------------------------------------------------------------------
Boolean TDBElementTokenIterator::Contains(TAbstractScriptableObject* objectToTestForMembership)
{
	Boolean isContained = false;
	
	for(this->Reset(); this->More() && (isContained == false); this->Next())
	{
		AConst<TDBRecord> current = fIterator.Current();

		// isContained = current ???;	// 
	}
	
	return isContained;
} // TDBElementTokenIterator::Contains

#endif

//----------------------------------------------------------------------------------------
// TDBElementTokenIterator::AccessBySearchSpec: 
//----------------------------------------------------------------------------------------
void TDBElementTokenIterator::AccessBySearchSpec(TTransaction* t, TAbstractCollector* collector, DescType desiredClass, TAbstractSearchSpec* searchSpec)
	{
	TAbstractScriptableObject* resultToken = nil;	
	TAbstractScriptableObject* token = nil;

	NOREGISTER(token);
		
	Boolean iterationDirection = collector->CollectorRequest(t, kSearchShouldIterateBackwards) ? kBackwardIteration : kForwardIteration;
	for(this->Reset(t, iterationDirection); this->More(t) && (collector->CollectorRequest(t, kCollectionIsFull) == false); this->Next(t))
		{
		TRY
			{
			if(token == nil)
				token = this->Current(t);
			else
				((TDBElementToken*)token)->AssignDifferentElement(fIterator.Current()->DBElementCursor());
			if(searchSpec->Compare(t, token) && (token->DerivedFromOSLClass(t, desiredClass) == true))
				{
				collector->AddToCollection(token);
				token = nil;
				}
			}
		EXCEPT
			{
			}
		ENDTRY
		}

	if(token != nil)
		token->DisposeDesignator();
	} // TDBElementTokenIterator::AccessBySearchSpec
