

#include "HFSBackingStore.h"

//--------------------------------------------------------------------------------
// THFSBackingStore::THFSBackingStore
//--------------------------------------------------------------------------------
THFSBackingStore::THFSBackingStore()
{
	fRefNum = kFileNotOpen;
	fFileEOF = 0;
	//  set up an arbitrary FSSpec here in the temporary items folder
	this->OpenFile();
} // THFSBackingStore::THFSBackingStore

//--------------------------------------------------------------------------------
// THFSBackingStore::THFSBackingStore
//--------------------------------------------------------------------------------
THFSBackingStore::THFSBackingStore(FSSpec& fileSpec)
{
	fRefNum = kFileNotOpen;
	fFileEOF = 0;
	fFileSpec = fileSpec;
	this->OpenFile();
} // THFSBackingStore::THFSBackingStore

//--------------------------------------------------------------------------------
// THFSBackingStore::~THFSBackingStore
//--------------------------------------------------------------------------------
THFSBackingStore::~THFSBackingStore()
{
	this->CloseFile();
} // THFSBackingStore::~THFSBackingStore

//--------------------------------------------------------------------------------
// THFSBackingStore::CanSaveDocument
//--------------------------------------------------------------------------------
Boolean THFSBackingStore::CanSaveDocument()
{
	//
	// Should also test to see if the file or volume is locked.
	//
	return TAbstractBackingStore::CanSaveDocument() && (fRefNum != kFileNotOpen);
}

//--------------------------------------------------------------------------------
// THFSBackingStore::Read
//--------------------------------------------------------------------------------
void THFSBackingStore::Read(void* bufferStart, long startByteInFile, long numberOfBytes)
{
	//
	// Should fail if the file is not open
	//
	if(fRefNum != kFileNotOpen)
	{
		long bytesToRead = numberOfBytes;
		FailErr(SetFPos(fRefNum, fsFromStart, startByteInFile));
		FailErr(FSRead(fRefNum, &bytesToRead, bufferStart));
		ASSERT(bytesToRead == numberOfBytes);
	}
} // THFSBackingStore::Read

//--------------------------------------------------------------------------------
// THFSBackingStore::Write
//--------------------------------------------------------------------------------
void THFSBackingStore::Write(void* bufferStart, long startByteInFile, long numberOfBytes)
{
	//
	// Should fail if the file is not open
	//
	if(this->CanSaveDocument())
	{
		long bytesToWrite = numberOfBytes;

		if(startByteInFile + numberOfBytes > fFileEOF)
		{
			long newEOF = startByteInFile + numberOfBytes;
			FailErr(SetEOF(fRefNum, newEOF));
			fFileEOF = newEOF;
		}

		FailErr(SetFPos(fRefNum, fsFromStart, startByteInFile));
		FailErr(FSWrite(fRefNum, &bytesToWrite, bufferStart));
		ASSERT(bytesToWrite == numberOfBytes);
	}
} // THFSBackingStore::Write

//--------------------------------------------------------------------------------
// THFSBackingStore::OpenFile
//--------------------------------------------------------------------------------
void THFSBackingStore::OpenFile()
{
	if(fRefNum == kFileNotOpen)
	{
		//
		// Should create the file if it doesn't exist
		// If the file does exist, then we should require
		// that its file type is what we expect it to be.
		//
		
		//
		// Currently we always open a file read/write.  It
		// might be nice to allow read-only access to a file
		// as well, but currently there's no way to specify that.
		//
		FailErr(FSpOpenDF(&fFileSpec, fsRdWrPerm, &fRefNum));

		FailErr(GetEOF(fRefNum, &fFileEOF));
	}
} // THFSBackingStore::OpenFile

//--------------------------------------------------------------------------------
// THFSBackingStore::CloseFile
//--------------------------------------------------------------------------------
void THFSBackingStore::CloseFile()
{
	//
	// We never want 'Close' to fail.
	//
	if(fRefNum != kFileNotOpen)
	{
		FSClose(fRefNum);
		fRefNum = kFileNotOpen;
	}
	FlushVol("\p", fFileSpec.vRefNum);
} // THFSBackingStore::CloseFile

//--------------------------------------------------------------------------------
// THFSBackingStore::DocumentName
//--------------------------------------------------------------------------------
void THFSBackingStore::DocumentName(TUpdataDataReference& name)
{
	name.CopyFrom(TConstDataReference(typeChar, (Ptr)&fFileSpec.name[1], fFileSpec.name[0]));
}
