
#ifndef __ABSTRACTDATA__
#define __ABSTRACTDATA__


//
// Enumerations for comparison results
//
enum CompareEnumeration
{
	kSecondObjectComesBefore = -1,
	kObjectKeysEqual,
	kSecondObjectComesAfter
};

enum
{
	kAnyType = '****'
};

class TConstDataReference;

//================================================================================
// Class TAbstractCompareBehavior
//================================================================================
class TAbstractCompareBehavior
{
private:
	TAbstractCompareBehavior*			fNextBehavior;

public:
	TAbstractCompareBehavior();
	~TAbstractCompareBehavior() { delete fNextBehavior; }

	void SetNextBehavior(TAbstractCompareBehavior* nextBehavior) { fNextBehavior = nextBehavior; }
	
	virtual CompareEnumeration CompareDynamicBehavior(const TConstDataReference& object1, const TConstDataReference& object2) const;
	virtual Boolean ContainsDynamicBehavior(const TConstDataReference& object1, const TConstDataReference& object2) const;
};


//================================================================================
// Class TConstDataReference
//================================================================================
class TConstDataReference
{
private:
	static TAbstractCompareBehavior* fFirstBehavior;
	
protected:
	long				fDataType;
	char*				fData;
	long				fDataLength;

public:
	TConstDataReference(long dataType, const char* data, long dataLength) : fDataType(dataType), fData((char*)data), fDataLength(dataLength) {}

	long DataType() const { return fDataType; }
	const char* Data() const { return fData; }
	long DataLength() const { return fDataLength; }
	
	long CopyTo(char* destination, long maxBytesToCopy) const;

	static void AddCompareBehavoir(TAbstractCompareBehavior* behavior);
	
	static CompareEnumeration Compare(const TConstDataReference& object1, const TConstDataReference& object2);
	static CompareEnumeration CompareDefaultBehavior(const TConstDataReference& object1, const TConstDataReference& object2);

	static Boolean ContainsDefaultBehavior(const TConstDataReference& object1, const TConstDataReference& object2);

	CompareEnumeration Compare(const TConstDataReference& object2) const;
	
	Boolean Contains(const TConstDataReference& object2) const;
};

//================================================================================
// Class TUpdataDataReference
//================================================================================
class TUpdataDataReference : public TConstDataReference
{
private:
	long				fMaxLength;
	
public:
	TUpdataDataReference(long dataType, char* data, long dataLength, long maxLength) : TConstDataReference(dataType, data, dataLength), fMaxLength(maxLength) {}

	long MaxLength() { return fMaxLength; }
	Boolean CopyFrom(const TConstDataReference& source, Boolean allowDataToClip = false);

	void SetDataType(long newType) { fDataType = newType; }
	void SetDataLength(long newLength) { fDataLength = newLength; }
};

#endif
