/*
	File:		Int64.h

	Contains:	xxx put contents here xxx

	Written by:	xxx put writers here xxx

	Copyright:	 1994-1995 by Apple Computer, Inc., all rights reserved.

		 <2>	 3/20/95	andy	
*/

#ifndef Int64_h
#define Int64_h

#ifndef REZ

#include <Types.h>

// ga
#define typeInt64		'in64'

class TIcon;
class Int64;

enum 
	{
	kInt64CanNotFitIn32Bits = 0x7FFFFFFF,
	
	kRemoveSize				=	true,
	kDontRemoveSize			=	false,

	typeOtherIsInt64		= 0,
	typeOtherIsInt			= 128,
	typeOtherIsULong 		= 64,
	typeOtherIsLong			= 32,
	typeMask				= typeOtherIsInt64 | typeOtherIsInt | typeOtherIsULong | typeOtherIsLong,
	
	verbMask				= 0xF,
	verbAdd					= 0,
	verbSubtract			= 1,
	verbMultiply			= 2,
	verbDivide				= 3,
	verbShiftRight			= 4,
	verbShiftLeft			= 5,
	verbBitAnd				= 6,
	verbAssign				= 8,

	
	cmpMask					= verbMask,
	cmpNotEqual				= 0,
	cmpEqual				= 1,
	cmpGreaterOrEqual		= 2,
	cmpLess					= 3,
	cmpLessOrEqual			= 4,
	cmpGreater				= 5,

	opAssignInt64				= verbAssign 			| typeOtherIsInt64,
	opAddInt64					= verbAdd 				| typeOtherIsInt64,
	opSubtractInt64				= verbSubtract 			| typeOtherIsInt64,
	opMultiplyInt64				= verbMultiply 			| typeOtherIsInt64,
	opDivideInt64				= verbDivide 			| typeOtherIsInt64,
	opShiftRightInt64			= verbShiftRight 		| typeOtherIsInt64,
	opShiftLeftInt64			= verbShiftLeft 		| typeOtherIsInt64,

	opAssignLong				= verbAssign 			| typeOtherIsLong,
	opAddLong					= verbAdd 				| typeOtherIsLong,
	opSubtractLong				= verbSubtract 			| typeOtherIsLong,
	opMultiplyLong				= verbMultiply 			| typeOtherIsLong,
	opDivideLong				= verbDivide 			| typeOtherIsLong,
	opShiftRightLong			= verbShiftRight 		| typeOtherIsLong,
	opShiftLeftLong				= verbShiftLeft 		| typeOtherIsLong,

	opAssignUnsignedLong		= verbAssign 			| typeOtherIsULong,
	opAddUnsignedLong			= verbAdd 				| typeOtherIsULong,
	opSubtractUnsignedLong		= verbSubtract 			| typeOtherIsULong,
	opMultiplyUnsignedLon		= verbMultiply 			| typeOtherIsULong,
	opDivideUnsignedLong		= verbDivide 			| typeOtherIsULong,
	
	opAssignInt					= verbAssign 			| typeOtherIsInt,
	opAddInt					= verbAdd 				| typeOtherIsInt,
	opSubtractInt				= verbSubtract 			| typeOtherIsInt,
	opMultiplyInt				= verbMultiply 			| typeOtherIsInt,
	opDivideInt					= verbDivide 			| typeOtherIsInt,

	cmpNotEqualInt64			= cmpNotEqual 			| typeOtherIsInt64,
	cmpEqualInt64				= cmpEqual 				| typeOtherIsInt64,
	cmpGreaterOrEqualInt64		= cmpGreaterOrEqual 	| typeOtherIsInt64,
	cmpLessInt64				= cmpLess 				| typeOtherIsInt64,
	cmpLessOrEqualInt64			= cmpLessOrEqual 		| typeOtherIsInt64,
	cmpGreaterInt64				= cmpGreater 			| typeOtherIsInt64,

	cmpNotEqualLong				= cmpNotEqual 			| typeOtherIsLong,
	cmpEqualLong				= cmpEqual 				| typeOtherIsLong,
	cmpGreaterOrEqualLong		= cmpGreaterOrEqual 	| typeOtherIsLong,
	cmpLessLong					= cmpLess 				| typeOtherIsLong,
	cmpLessOrEqualLong			= cmpLessOrEqual 		| typeOtherIsLong,
	cmpGreaterLong				= cmpGreater 			| typeOtherIsLong,

	cmpNotEqualUnsignedLong		= cmpNotEqual 			| typeOtherIsULong,
	cmpEqualUnsignedLong		= cmpEqual 				| typeOtherIsULong,
	cmpGreaterOrEqualUnsignedLong= cmpGreaterOrEqual 	| typeOtherIsULong,
	cmpLessUnsignedLong			= cmpLess 				| typeOtherIsULong,
	cmpLessOrEqualUnsignedLong	= cmpLessOrEqual 		| typeOtherIsULong,
	cmpGreaterUnsignedLong		= cmpGreater 			| typeOtherIsULong
	};

// I used my own defin here in so that all of the math is based on 
// the one assumption of 32 bit longs. Didn't want this to partially
// work for other sizes.
#define MAXULONG ((2 ^ 32) - 1)


//========================================================================================
// CLASS Int64
//
// int is the natural type for the value 0 {ARM, pg 324}, so TWide
// has a number of conversion operators from integer types so that
// there is never any ambiguity {upon which the compiler would barf!}.
//
// For instance, if there were no TWide(int) the compiler would try to
// match TWide aWide = 0; with both TWide(long) and TWide(unsigned long),
// both of which are conversions, yielding an ambiguous state.
//========================================================================================

class Int64
	{

	//
	// --- Methods -------------------------------------------------------------------------------------
	//
public:
			Int64();
			Int64(int i);
			Int64(long l);
			Int64(unsigned long l);
			Int64(unsigned long high,unsigned long low);

			operator long() const;
			operator unsigned long() const;
			operator double() const;

			unsigned long High32();
			unsigned long Low32();

	Int64&				operator =(const Int64&);
	Int64&				operator =(int);				// to convert from zero
	Int64&				operator =(long);				// to convert non-zero signed values
	Int64&				operator =(unsigned long);		// to convert non-zero unsigned values
	
	Int64				operator +(const Int64&) const;
	Int64				operator -(const Int64&) const;
	Int64				operator *(const Int64&) const;
	Int64				operator /(const Int64&) const;
	Int64				operator >>(const Int64&) const;
	Int64				operator <<(const Int64&) const;

	//
	// Because of the way Int64's are passed around and made into temporaries these 
	// operate and assign functions are more efficient than the above ones.
	//
	Int64&				operator +=(const Int64&);
	Int64&				operator -=(const Int64&);
	Int64&				operator >>=(const Int64&);
	Int64&				operator <<=(const Int64&);
	Int64&				operator *=(const Int64&);
	Int64&				operator &=(const Int64&);

	Boolean 			operator !=(const Int64&) const;
	Boolean 			operator ==(const Int64&) const;
	Boolean 			operator >=(const Int64&) const;
	Boolean 			operator <(const Int64&) const;
	Boolean 			operator <=(const Int64&) const;
	Boolean 			operator >(const Int64&) const;
		
	Boolean				FitsInLong() const;
	Boolean				FitsInUnsignedLong() const;

protected:
	static const Int64* IntuitOtherOperand(short operation, const void* other, Int64* storage);
	Int64				Operation(short operation, const void* other) const;
	Boolean				Comparison(short comparison, const void* otherP) const;

	//
	// --- Fields -------------------------------------------------------------------------------------
	//
	long				fHigh32;
	unsigned long		fLow32;
	};

//inline Boolean operator < (long l, const Int64& i64) { return (i64 > l); }

inline unsigned long Int64::High32() 	{ return fHigh32; 	}
inline unsigned long Int64::Low32() 	{ return fLow32; 	}

//
// Inlines
//

//----------------------------------------------------------------------------------------
// Int64::FitsInLong: 
//
// to legally crunch a signed 63 into a signed 31 bit number, the high 32 bits
// must be only sign extension { zero or negative one }, and the high bit of the low
// 32 bits must equal the sign, otherwise that 32nd bit is being used to store a
// component of the value.
//----------------------------------------------------------------------------------------
inline Boolean	Int64::FitsInLong() const
	{
	Boolean result = false;					// assume large to be safe
	
	if (fHigh32 == 0)						// could be a  positive number, 
		result = (*(long*)&fLow32 >= 0);	// can it fit in 31 bits?
	else if (fHigh32 == -1)					// could be a negative number
		result = (*(long*)&fLow32 < 0);		// can it fit in 31 bits?
	
	// the high 32 bits was not a sign extension so this quantity can't fit in 32
	
	return result;
	}

//----------------------------------------------------------------------------------------
// Int64::FitsInUnsignedLong: 
//----------------------------------------------------------------------------------------
inline Boolean	Int64::FitsInUnsignedLong() const
	{
	
	//
	// if high 32 is a sign extension then true
	//
	// notice that wrapping around is made legal
	// I think that this is good, but I could be posessed by the devil
	// -Dave0
	//
	return (fHigh32 == 0) || (fHigh32 == -1);
	}

//----------------------------------------------------------------------------------------
// Int64::=: 
//
// WARNING: DO NOT declare an Int64 inside this method and assign another Int64 to it,
//			or you'll recurse forever.  Me and Max, we've been there.  Done that. -andy
//----------------------------------------------------------------------------------------
inline Int64& Int64::operator =(const Int64& amount)			
	{
	fHigh32 = amount.fHigh32;
	fLow32  = amount.fLow32;

	return *this;
	} 

//----------------------------------------------------------------------------------------
// Int64::=: 
//----------------------------------------------------------------------------------------
inline Int64& 	Int64::operator =(int amount)					
	{ 
	fLow32  = amount;
	fHigh32 = 0;

	if (amount < 0)
		fHigh32 =  -1;

	return *this;
	} 
	

//----------------------------------------------------------------------------------------
// Int64::=: 
//----------------------------------------------------------------------------------------
inline Int64& 	Int64::operator =(long amount)					
	{ 
	fLow32  = amount;
	fHigh32 = 0;

	if (amount < 0)
		fHigh32 =  -1;

	return *this;
	}
	

//----------------------------------------------------------------------------------------
// Int64::=: 
//----------------------------------------------------------------------------------------
inline Int64& 	Int64::operator =(unsigned long amount)			
	{ 
	fLow32  = amount;
	fHigh32 = 0;

	return *this;
	} 

//----------------------------------------------------------------------------------------
// Int64::Int64: 
//----------------------------------------------------------------------------------------
inline Int64::Int64()					
	{
	;
	}
	
//----------------------------------------------------------------------------------------
// Int64::Int64: 
//----------------------------------------------------------------------------------------
inline Int64::Int64(int amount)		
	{
	fLow32  = amount;
	fHigh32 = 0;

	if (amount < 0)
		fHigh32 =  -1;
	}


//----------------------------------------------------------------------------------------
// Int64::Int64: 
//----------------------------------------------------------------------------------------
inline Int64::Int64(long amount)			
	{
	fLow32  = amount;
	fHigh32 = 0;

	if (amount < 0)
		fHigh32 =  -1;
	} 


//----------------------------------------------------------------------------------------
// Int64::Int64: 
//----------------------------------------------------------------------------------------
inline Int64::Int64(unsigned long amount)
	{
	fHigh32 = 0;
	fLow32  = amount;
	}
	

//----------------------------------------------------------------------------------------
// Int64::Int64: 
//----------------------------------------------------------------------------------------
inline Int64::Int64(unsigned long high,unsigned long low)	
	{
	fHigh32 = high;
	fLow32 = low;
	}


//----------------------------------------------------------------------------------------
// Int64::long: 
//----------------------------------------------------------------------------------------
inline Int64::operator long() const
	{
	if (FitsInLong())
		return fLow32;
	else
		return kInt64CanNotFitIn32Bits;
	}


//----------------------------------------------------------------------------------------
// Int64::long: 
//----------------------------------------------------------------------------------------
inline Int64::operator unsigned long() const
	{
	if (FitsInUnsignedLong())
		return fLow32;
	else
		return kInt64CanNotFitIn32Bits;
	}


//----------------------------------------------------------------------------------------
// Int64::double: 
//----------------------------------------------------------------------------------------
inline Int64::operator double() const
	{
	double result;
	
	result  = fHigh32;
	result *= 4294967296.0;					// 2 ^ 32 (done on Tom's HP calculator, no less)
	
	result += double(fLow32);
	
	return result;
	}	

//----------------------------------------------------------------------------------------
// Int64::+=: 
//----------------------------------------------------------------------------------------
inline Int64& 	Int64::operator +=(const Int64& amount)			
	{ 
	Int64 result;
	
	result  = Operation(opAddInt64, &amount); 		
	fHigh32 = result.High32();
	fLow32  = result.Low32();

	return *this;
	} // Int64::+= 
	

//----------------------------------------------------------------------------------------
// Int64::-=: 
//----------------------------------------------------------------------------------------
inline Int64&	Int64::operator -=(const Int64& amount)			
	{ 
	Int64 result;
	
	result  = Operation(opSubtractInt64, &amount); 	
	fHigh32 = result.High32();
	fLow32  = result.Low32();

	return *this;
	} // Int64::-= 
	

//----------------------------------------------------------------------------------------
// Int64::>>=: 
//----------------------------------------------------------------------------------------
inline Int64& 	Int64::operator >>=(const Int64& amount)		
	{ 
	Int64 result;
	
	result  = Operation(opShiftRightInt64, &amount);
	fHigh32 = result.High32();
	fLow32  = result.Low32();

	return *this;
	} // Int64::>>= 
	

//----------------------------------------------------------------------------------------
// Int64::<<=: 
//----------------------------------------------------------------------------------------
inline Int64& 	Int64::operator <<=(const Int64& amount)		
	{ 
	Int64 result;
	
	result  = Operation(opShiftLeftInt64, &amount); 
	fHigh32 = result.High32();
	fLow32  = result.Low32();

	return *this;
	} // Int64::<<= 
	

//----------------------------------------------------------------------------------------
// Int64::*=: 
//----------------------------------------------------------------------------------------
inline Int64& 	Int64::operator *=(const Int64& amount)			
	{ 
	Int64 result;
	
	result  = Operation(opMultiplyInt64, &amount); 	
	fHigh32 = result.High32();
	fLow32  = result.Low32();

	return *this;
	} // Int64::*= 


//----------------------------------------------------------------------------------------
// Int64::operator &=: 
//----------------------------------------------------------------------------------------
inline Int64& Int64::operator &=(const Int64& amount)			
	{
	fHigh32 	= fHigh32 & amount.fHigh32;
	fLow32		= fLow32  & amount.fLow32;

	return *this;
	}


inline Int64	Int64::operator +(const Int64& amount) const	{ return Operation(opAddInt64, &amount); 					}
inline Int64 	Int64::operator -(const Int64& amount) const	{ return Operation(opSubtractInt64, &amount); 				}
inline Int64	Int64::operator *(const Int64& amount) const	{ return Operation(opMultiplyInt64, &amount); 				}
inline Int64 	Int64::operator /(const Int64& amount) const	{ return Operation(opDivideInt64, &amount); 				}
inline Int64 	Int64::operator >>(const Int64& amount) const	{ return Operation(opShiftRightInt64, &amount); 			}
inline Int64 	Int64::operator <<(const Int64& amount) const	{ return Operation(opShiftLeftInt64, &amount); 				}

inline Boolean 	Int64::operator !=(const Int64& b) const		{ return Comparison(cmpNotEqualInt64, &b); 					}
inline Boolean 	Int64::operator ==(const Int64& b) const		{ return Comparison(cmpEqualInt64, &b); 					}
inline Boolean 	Int64::operator >=(const Int64& b) const		{ return Comparison(cmpGreaterOrEqualInt64, &b); 			}
inline Boolean 	Int64::operator <(const Int64& b) const			{ return Comparison(cmpLessInt64, &b); 						}
inline Boolean 	Int64::operator <=(const Int64& b) const		{ return Comparison(cmpLessOrEqualInt64, &b); 				}
inline Boolean 	Int64::operator >(const Int64& b) const			{ return Comparison(cmpGreaterInt64, &b); 					}


#endif // REZ
#endif // Int64_h
