


#ifndef __ABSTRACTITERATOR__
#define __ABSTRACTITERATOR__

//
// MoreAEM is needed for declaration of TDescriptor
//
#include "MoreAEM.h"

//
// Iteration direction
//
enum
{
	kForwardIteration = 0,
	kBackwardIteration
};

class TAbstractScriptableObject;
class TAbstractCollector;
class TAbstractSearchSpec;
class TTransaction;

//========================================================================================
// Class TAbstractObjectIterator
//========================================================================================
class TAbstractObjectIterator
{
public:
	virtual ~TAbstractObjectIterator();
	
	//
	// Call Release to free an iterator; this way, subclasses of
	// TAbstractObjectIterator may be reference-counted, if desired.
	// It would be nice to make ~TAbstractObjectIterator protected
	// to prevent clients from calling 'delete' directly, but
	// that also prevents instantiation of stack-based iterators,
	// which isn't good.
	//
	virtual void						Release();
	
	//
	// Interface to code to:
	//
	virtual void						Reset(TTransaction* t, Boolean iterationDirection = kForwardIteration) = 0;
	virtual Boolean						More(TTransaction*) const = 0;
	virtual void						Next(TTransaction*) = 0;
	virtual TAbstractScriptableObject*	Current(TTransaction*) = 0;
	virtual void						RemoveCurrent(TTransaction*);	// Default implementation fails

	//
	// Need to rename this routine
	//
	virtual Boolean						Contains(TTransaction* t, TAbstractScriptableObject* objectToTestForMembership);

	//
	// Override these methods to improve performance:
	//
	virtual Boolean						CurrentDerivedFromOSLClass(TTransaction* t, DescType objectClass);
	virtual long						CountElements(TTransaction* t, DescType desiredClass);
	virtual TAbstractScriptableObject*	GetIndexedElement(TTransaction* t, DescType desiredClass, long index);
	virtual TAbstractScriptableObject*	GetNamedElement(TTransaction* t, DescType desiredClass, TDescriptor nameDesc);
	virtual void						AccessBySearchSpec(TTransaction* t, TAbstractCollector* collector, DescType desiredClass, TAbstractSearchSpec* searchSpec);
	virtual void						SearchDeep(TTransaction* t, TAbstractCollector* collector, DescType desiredClass, TAbstractSearchSpec* searchSpec);
	void								RecursiveSearchDeep(TTransaction* t, TAbstractCollector* collector, DescType desiredClass, TAbstractSearchSpec* searchSpec);
};

//========================================================================================
// Class TSingleObjectIterator
//========================================================================================
class TSingleObjectIterator : public TAbstractObjectIterator
{
private:
	TAbstractScriptableObject*			fObject;
	TAbstractScriptableObject*			fCurrent;
	
public:
	virtual ~TSingleObjectIterator();
	TSingleObjectIterator(TAbstractScriptableObject* object) : fObject(object), fCurrent(object) {}
	
	//
	// Interface to code to:
	//
	virtual void						Reset(TTransaction* t, Boolean iterationDirection = kForwardIteration);
	virtual Boolean						More(TTransaction*) const;
	virtual void						Next(TTransaction*);
	virtual TAbstractScriptableObject*	Current(TTransaction*);

	//
	// Performance enhancement:
	//
	virtual long						CountElements(TTransaction* t, DescType desiredClass);
	virtual TAbstractScriptableObject*	GetIndexedElement(TTransaction* t, DescType desiredClass, long index);
};

//========================================================================================
// CLASS TCollectionIterator
//========================================================================================

class TCollectionIterator : public TAbstractObjectIterator
{
private:
	TAbstractObjectIterator**			fIterCollection;
	long								fItersInCollection;
	long								fCollectionSize;
	long								fCurrentIter;
	Boolean								fDirection;
	
public:
	virtual ~TCollectionIterator();
	TCollectionIterator() :
		fIterCollection(nil),
		fItersInCollection(0),
		fCollectionSize(0),
		fCurrentIter(0),
		fDirection(kForwardIteration) {}

	void								AddIterator(TAbstractObjectIterator* iter);
	void								AddObject(TTransaction* t, TAbstractScriptableObject* object);
	void								AddElementsOfObject(TTransaction* t, TAbstractScriptableObject* object);
	TAbstractObjectIterator*			CurrentIterator() const;

	//
	// Interface to code to:
	//
	virtual void						Reset(TTransaction* t, Boolean iterationDirection = kForwardIteration);
	virtual Boolean						More(TTransaction*) const;
	virtual void						Next(TTransaction*);
	virtual TAbstractScriptableObject*	Current(TTransaction*);

	//
	// Performance enhancement:
	//
	virtual Boolean						CurrentDerivedFromOSLClass(TTransaction* t, DescType objectClass);
	virtual void						SearchDeep(TTransaction* t, TAbstractCollector* collector, DescType desiredClass, TAbstractSearchSpec* searchSpec);
};

#endif
