/*
	File:		AbstractWindowToken.h

	Contains:	xxx put contents here xxx

	Written by:	Andy Nicholas, Greg Anderson, Tom Conrad, Chris Bingham, Georgiann Puckett, John Thompson-Rohrlich

	Copyright:	 1994-1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <9>	 9/26/95	bing	Include AbstractDesignator.h.
		 <8>	 8/17/95	ga		Source merge with db+
		 <7>	  6/6/95	ga		Combine ObjectClass and ObjectsRecordedClass; get rid of
									CreatePropertyFromDescription and GenericFinderProperty
		 <6>	 2/13/95	ga		Nothing better to do, so I thought I'd replace
									IMPLEMENT/DECLAREOBJECT with Implement/DeclareClassData. Fixed a
									lot of scripting problems too.
		 <5>	  2/8/95	jtr		(ga)Remove DesignatorIdentifier
		 <4>	  2/3/95	ga		Add new class IDs for classes with no properties or resources
		 <3>	  6/9/94	bing	Use GetWindowRef and GetWindowPort.
		 <2>	  6/8/94	bing	ToolboxWindow() -> WindowRef() to conform to TWindow.
		 <1>	  3/8/94	ga		first checked in
		 <4>	 2/24/94	ga		Break out TFinderWindowToken now that it more or less works
		 <3>	 2/22/94	ga		Fix xLC build
		 <2>	 2/21/94	ga		Put initial implementation of TFinderWindowToken here
		 <1>	 2/16/94	ga		first checked in

*/

#ifndef AbstractWindowToken_h
#define AbstractWindowToken_h
#define __ABSTRACTWINDOWTOKEN__

#define kIsModal		true
#define kIsResizable	true
#define kIsZoomable		true
#define kIsFloating		true
#define kHasTitleBar	true

//
// AbstractDesignator.h is needed because
// TAbstractDesignator is the base class of TAbstractWindowToken
//
#ifndef AbstractDesignator_h
#include "AbstractDesignator.h"
#endif

//
// MoreWindows.h is needed for definition of WindowRef.
//
#ifndef MoreWindows_h
#include "MoreWindows.h"
#endif

class TAbstractWindowToken : public TAbstractDesignator
	{
public:
	DeclareClassData(TAbstractWindowToken, TAbstractDesignator);

										TAbstractWindowToken();
	virtual								~TAbstractWindowToken();
	
	void								IAbstractWindowToken(TAbstractScriptableObject* windowOwner, Boolean isModal = !kIsModal, Boolean isResizable = kIsResizable, Boolean isZoomable = kIsZoomable, Boolean isFloating = !kIsFloating, Boolean hasTitleBar = kHasTitleBar);
	
	virtual DescType					ObjectClass(Boolean recordedClass = false);
	virtual DescType					WindowClass();
	virtual Boolean						DerivedFromOSLClass(DescType objectClass);

	virtual Boolean						Exists();
	virtual TAbstractScriptableObject*	ParentObject();

	virtual void						MakeKeyDataForSelf(DescType& keyForm, TDescriptor& keyData);

//	virtual TAbstractScriptableObject*	AccessByProperty(DescType propertyIdentifier);

	virtual TDescriptor					GetProperty(DescType propertyName, DescType desiredType, unsigned long additionalInfo = 0);
	virtual void						SetProperty(TTransaction* transaction, DescType propertyName, TDescriptor& data, unsigned long additionalInfo = 0);

	virtual WindowRef					GetWindowRef();

	virtual Rect						GetWindowBounds();
	virtual void						SetWindowBounds(Rect& newBounds);
	virtual void						MoveWindow(short newH, short newV);
	virtual void						SizeWindow(short newW, short newH);

	long								ItemIndex();
	void								SetItemIndex(long newIndex);

	Boolean								HasCloseBox();
	Boolean								HasTitleBar();
	Boolean								IsFloating();
	Boolean								IsModal();
	Boolean								IsResizable();
	Boolean								IsZoomable();
	virtual Boolean						IsZoomed();
	virtual void						SetZoomed(Boolean newZoom);
	Boolean								IsVisible();
	void								SetVisible(Boolean newVisible);

protected:
	
	Boolean								fHasTitleBar;
	Boolean								fIsModal;
	Boolean								fIsResizable;
	Boolean								fIsFloating;
	Boolean								fIsZoomable;
	
	};

#endif
