/*
	File:		ProxyToken.h

	Contains:	Proxies (base class for things such as the selection)

	Written by:	Greg Anderson

	Copyright:	 1993-1995 by Apple Computer, Inc., all rights reserved.

		 <5>	 6/23/95	ga		
*/


#ifndef __PROXYTOKEN__
#define __PROXYTOKEN__
#define ProxyToken_h

//
// AbstractDesignator.h is needed because
// TAbstractDesignator is the base class of TProxyToken
//
#include "AbstractDesignator.h"

//
// AbstractProperty.h is needed because
// TGenericProperty is the base class of TPropertyOfProxy
//
#include "AbstractProperty.h"

#define cProxy		'prxy'

class TProxyToken : public TAbstractDesignator
	{
public:
	DeclareSmallClassData(TProxyToken, TAbstractDesignator);

										TProxyToken();
	virtual								~TProxyToken();
	
	void								IProxyToken(TAbstractScriptableObject* objectOfDesignation = nil);

	virtual Boolean						DerivedFromOSLClass(TTransaction* t, DescType objectClass);
	virtual DescType					DefaultType(TTransaction* t, DescType propertyName);
	virtual DescType					BestType(TTransaction* t, DescType propertyName);

	virtual Boolean						Exists(TTransaction*);
	virtual TAbstractObjectIterator*	DirectObjectIterator(TTransaction*);

	virtual Boolean						PropertyAppliesToProxy(DescType propertyName);
	virtual TAbstractScriptableObject*	AccessByOrdinal(TTransaction* t, DescType desiredClass, DescType ordinal);
	virtual TAbstractScriptableObject*	AccessByProperty(TTransaction* t, DescType prop);

	// Event Handlers:

//	virtual TDescriptor					GetData(DescType requestedType);
//	virtual void						SetData(TDescriptor keyData);

	virtual TDescriptor					GetProperty(TTransaction* t, DescType propertyName, DescType desiredType, unsigned long additionalInfo = 0);
	virtual void						SetProperty(TTransaction* transaction, DescType propertyName, TDescriptor& data, unsigned long additionalInfo = 0);

	virtual TDescriptor					BuildObjectSpecifier(TTransaction*);
	virtual Boolean						CanReturnDataOfType(TTransaction* t, DescType propertyName, DescType desiredType);

	TPropertyDescription				LookupPropertyDesciption(TTransaction* t, DescType propertyIdentifier);

	virtual TDescriptor					AECommandDispatch(TTransaction* transaction, TAEvent ae, TAEvent reply, long aeCommandID, long auxInfo = 0);
	virtual TDescriptor					AECommand(TTransaction* transaction, TAEvent ae, TAEvent reply, long aeCommandID, TAbstractScriptableObject* auxObjects = nil, long auxInfo = 0);
	virtual TAbstractScriptableObject*	CreateNewElement(TTransaction* transaction, TAEvent ae, TAEvent reply, DescType newObjectClass, TDescriptor initialData, TDescriptor initialProperties, Boolean& usedInitialData, Boolean& usedInitialProperties);

protected:
	};

#endif
