/*
	File:		ScriptableObjectList.cp

	Contains:	xxx put contents here xxx

	Written by:	Andy Nicholas, Greg Anderson, Chris Bingham, John Rohrlich, Max McFarland, Paul Ossenbruggen, Mike Kobb

	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.

		 <1>	  6/6/95	ga		

*/

#include "ScriptableObjectList.h"
#include "AbstractScriptableObject.h"

//----------------------------------------------------------------------------------------
// TScriptableObjectListIterator::~TScriptableObjectListIterator
//----------------------------------------------------------------------------------------
TScriptableObjectListIterator::~TScriptableObjectListIterator()
{
} // TScriptableObjectListIterator::~TScriptableObjectListIterator

//----------------------------------------------------------------------------------------
// TScriptableObjectListIterator::Reset
//----------------------------------------------------------------------------------------
void TScriptableObjectListIterator::Reset(TTransaction*, Boolean iterationDirection /*= kForwardIteration*/)
{
	fDirection = iterationDirection;
	fCurrent = iterationDirection == kForwardIteration ? fList->First() : fList->Last();
}

//----------------------------------------------------------------------------------------
// TScriptableObjectListIterator::More
//----------------------------------------------------------------------------------------
Boolean TScriptableObjectListIterator::More(TTransaction*) const
{
	return fCurrent != nil;
}

//----------------------------------------------------------------------------------------
// TScriptableObjectListIterator::Next
//----------------------------------------------------------------------------------------
void TScriptableObjectListIterator::Next(TTransaction*)
{
	if(fDirection == kForwardIteration)
		fCurrent = fList->Next(fCurrent);
	else
		fCurrent = fList->Previous(fCurrent);
}

//----------------------------------------------------------------------------------------
// TScriptableObjectListIterator::Current
//----------------------------------------------------------------------------------------
TAbstractScriptableObject* TScriptableObjectListIterator::Current(TTransaction*)
{
	return fCurrent->CloneDesignator();
}

//----------------------------------------------------------------------------------------
// TScriptableObjectListIterator::CountElements
//----------------------------------------------------------------------------------------
long TScriptableObjectListIterator::CountElements(TTransaction* t, DescType desiredClass)
{
	if(desiredClass == typeWildCard)
		return fList->Count();
	else
		return TAbstractObjectIterator::CountElements(t, desiredClass);
}

//----------------------------------------------------------------------------------------
// TScriptableObjectListIterator::GetIndexedElement
//----------------------------------------------------------------------------------------
TAbstractScriptableObject* TScriptableObjectListIterator::GetIndexedElement(TTransaction* t, DescType desiredClass, long index)
{
	if(desiredClass == typeWildCard)
		return fList->Ith(index);
	else
		return TAbstractObjectIterator::GetIndexedElement(t, desiredClass, index);
}

//----------------------------------------------------------------------------------------
// TScriptableObjectListIterator::Contains
//----------------------------------------------------------------------------------------
Boolean TScriptableObjectListIterator::Contains(TTransaction*, TAbstractScriptableObject* objectToTestForMembership)
{
	return fList->Contains(objectToTestForMembership);
}


