/*
	File:		ScriptableObjectList.h

	Contains:	xxx put contents here xxx

	Written by:	Andy Nicholas, Greg Anderson, Chris Bingham, John Rohrlich, Max McFarland, Paul Ossenbruggen, Mike Kobb

	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.

		 <1>	  6/6/95	ga		

*/


#ifndef __SCRIPTABLEOBJECTLIST__
#define __SCRIPTABLEOBJECTLIST__

//
// List.h is needed to define TScriptableObjectList, TScriptableObjectLoop
// and FOREACHSCRIPTABLEOBJECT.
//
#include "List.h"

//
// AbstractIterator is needed for definition of TAbstractIterator
//
#include "AbstractIterator.h"

//
// Needed for definition of TAbstractScriptableObject so that
// we can use the DECLAREOBJECTLIST macro.
//
#include "AbstractScriptableObject.h"

DECLAREOBJECTLIST(TScriptableObjectList, TAbstractScriptableObject)
DECLARELOOP(TScriptableObjectLoop, TScriptableObjectList, TAbstractScriptableObject)

#define FOREACHSCRIPTABLEOBJECT(listvar) FOREACH(TScriptableObjectLoop, listvar, TAbstractScriptableObject, item)


//========================================================================================
// Class TScriptableObjectListIterator
//========================================================================================
class TScriptableObjectListIterator : public TAbstractObjectIterator
{
public:	
	virtual ~TScriptableObjectListIterator();

	TScriptableObjectListIterator(TScriptableObjectList* list) :
		fList(list), fCurrent(nil) {}
	
	//
	// Interface to code to:
	//
	virtual void						Reset(TTransaction* t, Boolean iterationDirection = kForwardIteration);
	virtual Boolean						More(TTransaction*) const;
	virtual void						Next(TTransaction*);
	virtual TAbstractScriptableObject*	Current(TTransaction*);

	//
	// Override these methods to improve performance:
	//
	virtual long						CountElements(TTransaction* t, DescType desiredClass);
	virtual TAbstractScriptableObject*	GetIndexedElement(TTransaction* t, DescType desiredClass, long index);
	virtual Boolean						Contains(TTransaction* t, TAbstractScriptableObject* objectToTestForMembership);

private:
	TScriptableObjectList*				fList;
	TAbstractScriptableObject*			fCurrent;
	Boolean								fDirection;
};

#endif
